import os

from pylons import config
from sqlalchemy.util import OrderedDict
from nose.tools import assert_equal

from ckanext.dgu.ons import importer
from ckan.tests import *


TEST_DIR = os.path.dirname(os.path.abspath(__file__))
SAMPLE_PATH = os.path.join(TEST_DIR, 'samples')
SAMPLE_FILEPATH_1 = os.path.join(SAMPLE_PATH, 'ons_hub_sample.xml')


class TestOnsData1:
    def setup(self):
        records_obj = importer.OnsDataRecords(SAMPLE_FILEPATH_1)
        self.records = [record for record in records_obj]
 
    def test_records(self):
        assert len(self.records) == 8
        record1 = self.records[0]
        r1_keys = record1.keys()
        r1_expected_keys = [u'title', u'link', u'description', u'pubDate', u'guid', u'hub:source-agency', u'hub:theme', u'hub:coverage', u'hub:designation', u'hub:geographic-breakdown', u'hub:language', u'hub:ipsv', u'hub:keywords', u'hub:altTitle', u'hub:nscl']
        r1_keys.sort(), r1_expected_keys.sort()
        assert r1_keys == r1_expected_keys

        expected_items = [
            (u'title', u'UK Official Holdings of International Reserves - December 2009'),
            (u'link', u'http://www.hm-treasury.gov.uk/national_statistics.htm'),
            (u'description', u"Monthly breakdown for government's net reserves, detailing gross reserves and gross liabilities."),
            (u'pubDate', u'Wed, 06 Jan 2010 09:30:00 GMT'),
            (u'guid', u'http://www.statistics.gov.uk/hub/id/119-36345'),
            (u'hub:source-agency', u'HM Treasury'),
            (u'hub:theme', u'Economy'),
            (u'hub:coverage', u'UK'),
            (u'hub:designation', u''),
            (u'hub:geographic-breakdown', u'UK and GB'),
            (u'hub:language', u'English'),
            (u'hub:ipsv', u'Economics and finance'),
            (u'hub:keywords', u'reserves;currency;assets;liabilities;gold;reserves;currency;assets;liabilities;gold'),
            (u'hub:altTitle', u'UK Reserves'),
            (u'hub:nscl', u'Economy;Government Receipts and Expenditure;Public Sector Finance;Economy;Government Receipts and Expenditure;Public Sector Finance')]
        for key, value in expected_items:
            assert record1[key] == value, 'Key %s: got %r but should be %r' % (key, record1[key], value)
        expected_keys = set([key for key, value in expected_items])
        keys = set(record1.keys())
        key_difference = expected_keys - keys
        assert not key_difference, key_difference


class TestOnsImporter:
    def test_split_title(self):
        expected_data = [
            (u'UK Official Holdings of International Reserves - December 2009',
             u'UK Official Holdings of International Reserves', u'December 2009'),
            (u'Probation statistics brief - July - September 2009',
             u'Probation statistics brief', u'July - September 2009'),
            (u'National Park, Parliamentary Constituency and Ward level mid-year population estimates (experimental) - Mid-2008',
             u'National Park, Parliamentary Constituency and Ward level mid-year population estimates (experimental)', u'Mid-2008'),
            ]
        for xml_title, title, date in expected_data:
            res_title, res_date = importer.OnsImporter._split_title(xml_title)
            assert_equal(title, res_title)
            assert_equal(date, res_date)

    def test_geo_coverage(self):
        coverage_tests = [
            ('UK', '111100: United Kingdom (England, Scotland, Wales, Northern Ireland)'),
            ('GB', '111000: Great Britain (England, Scotland, Wales)'),
            ('England and Wales', '101000: England, Wales'),
            ('England', '100000: England'),
            ('Wales', '001000: Wales'),
            ('Scotland', '010000: Scotland'),
            ('Northern Ireland', '000100: Northern Ireland'),
            ('International', '000001: Global'),
            ]
        for coverage_str, expected_coverage_db in coverage_tests:
            coverage_db = importer.OnsImporter._parse_geographic_coverage(coverage_str)
            assert_equal(coverage_db, expected_coverage_db)

    def test_department_agency(self):
        expected_results = [
            # (hub:source-agency value, department, agency)
            ('Information Centre for Health and Social Care', '', 'Information Centre for Health and Social Care'),
            ('Business, Innovation and Skills', 'Department for Business, Innovation and Skills', ''),
            ('Communities and Local Government', 'Department for Communities and Local Government', ''),
            ('Culture, Arts and Leisure (Northern Ireland)', 'Northern Ireland Executive', ''),
            ('Defence', 'Ministry of Defence', ''),
            ('Education', 'Department for Education', ''),
            ('Employment and Learning (Northern Ireland)', 'Northern Ireland Executive', ''),
            ('Energy and Climate Change', 'Department of Energy and Climate Change', ''),
            ('Enterprise, Trade and Investment (Northern Ireland)', 'Northern Ireland Executive', ''),
            ('Environment, Food and Rural Affairs', 'Department for Environment, Food and Rural Affairs', ''),
            ('Environment (Northern Ireland)', 'Northern Ireland Executive', ''),
            ('Finance and Personnel (Northern Ireland)', 'Northern Ireland Executive', ''),
            ('Food Standards Agency', 'Food Standards Agency', ''),
            ('Forestry Commission', 'Forestry Commission', ''),
            ('General Register Office for Scotland', '', 'General Register Office for Scotland'),
            ('Health, Social Service and Public Safety (Northern Ireland)', 'Northern Ireland Executive', ''),
            ('Health', 'Department of Health', ''),
            ('Health and Safety Executive', '', 'Health and Safety Executive'),
            ('Health Protection Agency', '', 'Health Protection Agency'),
            ('HM Revenue and Customs', 'Her Majesty\'s Revenue and Customs', ''),
            ('HM Treasury', 'Her Majesty\'s Treasury', ''),
            ('Home Office', 'Home Office', ''),
            ('Information Centre for Health and Social Care', '', 'Information Centre for Health and Social Care'),
            ('International Development', 'Department for International Development', ''),
            ('ISD Scotland (part of NHS National Services Scotland)', '', 'ISD Scotland (part of NHS National Services Scotland)'),
            ('Justice', 'Ministry of Justice', ''),
            ('National Treatment Agency', '', 'National Treatment Agency'),
            ('NHS National Services Scotland', '', 'NHS National Services Scotland'),
            ('Northern Ireland Statistics and Research Agency', '', 'Northern Ireland Statistics and Research Agency'),
            ('Office for National Statistics', 'UK Statistics Authority', 'Office for National Statistics'),
            ('Office of Qualifications and Examinations Regulation', '', 'Office of Qualifications and Examinations Regulation'),
            ('Office of the First and Deputy First Minister', 'Northern Ireland Executive', ''),
            ('Passenger Focus', '', 'Passenger Focus'),
            ('Regional Development (Northern Ireland)', 'Northern Ireland Executive', ''),
            ('Scottish Government', 'Scottish Government', ''),
            ('Social Development (Northern Ireland)', 'Northern Ireland Executive', ''),
            ('Transport', 'Department for Transport', ''),
            ('Welsh Assembly Government', 'Welsh Assembly Government', ''),
            ('Work and Pensions', 'Department for Work and Pensions', ''),
            ]
        for source_agency, expected_department, expected_agency in expected_results:
            department, agency = importer.OnsImporter._source_to_department(source_agency)
            assert_equal(department, expected_department or None)
            assert_equal(agency, expected_agency or None)
        
    def test_record_2_package(self):
        record = OrderedDict([
            (u'title', u'UK Official Holdings of International Reserves - December 2009'),
            (u'link', u'http://www.hm-treasury.gov.uk/national_statistics.htm'),
            (u'description', u"Monthly breakdown for government's net reserves, detailing gross reserves and gross liabilities."),
            (u'pubDate', u'Wed, 06 Jan 2010 09:30:00 GMT'),
            (u'guid', u'http://www.statistics.gov.uk/hub/id/119-36345'),
            (u'hub:source-agency', u'HM Treasury'),
            (u'hub:theme', u'Economy'),
            (u'hub:coverage', u'UK'),
            (u'hub:designation', u''),
            (u'hub:geographic-breakdown', u'UK and GB'),
            (u'hub:language', u'English'),
            (u'hub:ipsv', u'Economics and finance'),
            (u'hub:keywords', u'reserves;currency;assets;liabilities;gold;reserves;currency;assets;liabilities;gold'),
            (u'hub:altTitle', u'UK Reserves'),
            (u'hub:nscl', u'Economy;Government Receipts and Expenditure;Public Sector Finance;Economy;Government Receipts and Expenditure;Public Sector Finance')])
        ons_importer_ = importer.OnsImporter(filepath=SAMPLE_FILEPATH_1)
        package_dict = ons_importer_.record_2_package(record)
    
        expected_package_dict = OrderedDict([
            ('name', u'uk_official_holdings_of_international_reserves'),
            ('title', u'UK Official Holdings of International Reserves'),
            ('version', None),
            ('url', None),
            ('author', u"Her Majesty's Treasury"),
            ('author_email', None),
            ('maintainer', None),
            ('maintainer_email', None),
            ('notes', u"Monthly breakdown for government's net reserves, detailing gross reserves and gross liabilities.\n\nSource agency: HM Treasury\n\nLanguage: English\n\nAlternative title: UK Reserves"),
            ('license_id', u'uk-ogl'),
            ('tags', [u'assets', u'currency', u'economics-and-finance', u'economy', u'gold', u'government-receipts-and-expenditure', u'liabilities', u'public-sector-finance', u'reserves']),
            ('groups', ['ukgov']),
            ('resources', [OrderedDict([
                ('url', u'http://www.hm-treasury.gov.uk/national_statistics.htm'),
                ('description', u'December 2009 | hub/id/119-36345'),
                ])]),
            ('extras', OrderedDict([
                ('geographic_coverage', u'111100: United Kingdom (England, Scotland, Wales, Northern Ireland)'),
                ('geographical_granularity', u'UK and GB'),
                ('external_reference', u'ONSHUB'),
                ('temporal_granularity', u''),
                ('date_updated', u''),
                ('agency', u''),
                ('precision', u''),
                ('temporal_coverage_to', u''),
                ('temporal_coverage_from', u''),
                ('national_statistic', 'no'),
                ('update_frequency', 'monthly'),
                ('department', u"Her Majesty's Treasury"),
                ('import_source', 'ONS-ons_hub_sample.xml'),
                ('date_released', '2010-01-06'),
                ('categories', u'Economy'),
                ('series', u'UK Official Holdings of International Reserves'),
                ])),
            ])
        for key, value in expected_package_dict.items():
            if key != 'extras':
                assert_equal(package_dict[key], value)
            else:
                for key, value in expected_package_dict['extras'].items():
                    assert_equal(package_dict['extras'][key], value)
        expected_keys = set(expected_package_dict.keys())
        keys = set(package_dict.keys())
        key_difference = expected_keys - keys
        assert not key_difference, key_difference




import os

from nose.tools import assert_equal
from pylons import config
from sqlalchemy.util import OrderedDict

from ckanext.dgu.ons import importer
from ckanext.dgu.ons.loader import OnsLoader
from ckanext.tests.test_loader import TestLoaderBase, USER
from ckan import model
from ckan.tests import *
from ckan.tests.wsgi_ckanclient import WsgiCkanClient


TEST_DIR = os.path.dirname(os.path.abspath(__file__))
SAMPLE_PATH = os.path.join(TEST_DIR, 'samples')
SAMPLE_FILEPATH_TEMPLATE = os.path.join(SAMPLE_PATH, 'ons_hub_sample%s.xml')
def sample_filepath(id):
    return SAMPLE_FILEPATH_TEMPLATE % id


class TestOnsLoadBasic(TestLoaderBase):
    @classmethod
    def setup_class(self):
        super(TestOnsLoadBasic, self).setup_class()
        importer_ = importer.OnsImporter(sample_filepath(''))
        self.pkg_dicts = [pkg_dict for pkg_dict in importer_.pkg_dict()]

        self.loader = OnsLoader(self.testclient)
        self.res = self.loader.load_packages(self.pkg_dicts)
        assert self.res['num_errors'] == 0, self.res
        CreateTestData.flag_for_deletion([pkg_dict['name'] for pkg_dict in self.pkg_dicts])

    def test_0_search_options(self):
        field_keys = ['title', 'department']

        pkg_dict = {'title':'titleA',
                    'extras':{'department':'Department for Children, Schools and Families'}}
        opts = self.loader._get_search_options(field_keys, pkg_dict)
        assert_equal(opts, [{'department': 'Department for Children, Schools and Families', 'title': 'titleA'}, {'department': 'Department for Education', 'title': 'titleA'}])

        pkg_dict = {'title':'titleA',
                    'extras':{'department':'',
                              'agency':'SomeAgency'}}
        opts = self.loader._get_search_options(field_keys, pkg_dict)
        assert_equal(opts, [{'agency':'SomeAgency', 'title': 'titleA'}])

    def test_1_hub_id_extraction(self):
        def assert_id(description, expected_id):
            resource = {'description':description}
            result = self.loader._get_hub_id(resource)
            assert_equal(result, expected_id)
        assert_id("April 2009 data: Experimental Statistics | hub/id/119-46440",
                  "119-46440")

    def test_2_date_choose(self):
        def assert_id(date1, date2, earlier_or_later, expected_date_index):
            dates = (date1, date2)
            pkg0 = {'extras':{'date': date1}}
            result = self.loader._choose_date(pkg0, date2,
                                              earlier_or_later,
                                              'date')
            if not expected_date_index:
                assert_equal(result, expected_date_index)
            else:
                assert_equal(result, dates[expected_date_index - 1])
        assert_id('2010-12-01', '2010-12-02', 'earlier', 1)
        assert_id('2010-12-01', '2010-12-02', 'later', 2)
        assert_id('2010-12-02', '2010-12-01', 'earlier', 2)
        assert_id('2010-12-02', '2010-12-01', 'later', 1)
        assert_id('', '2010-12-02', 'earlier', 2)
        assert_id('2010-12-01', '', 'later', 1)
        assert_id('', '', 'earlier', None)

    def test_fields(self):
        q = model.Session.query(model.Package)
        names = [pkg.name for pkg in q.all()]
        pkg1 = model.Package.by_name(u'uk_official_holdings_of_international_reserves')
        cereals = model.Package.by_name(u'cereals_and_oilseeds_production_harvest')
        custody = model.Package.by_name(u'end_of_custody_licence_release_and_recalls')
        probation = model.Package.by_name(u'probation_statistics_brief')
        assert pkg1, names
        assert cereals, names
        assert custody, names
        assert probation, names
        assert pkg1.title == 'UK Official Holdings of International Reserves', pkg1.title
        assert pkg1.notes.startswith("Monthly breakdown for government's net reserves, detailing gross reserves and gross liabilities."), pkg1.notes
        assert len(pkg1.resources) == 1, pkg1.resources
        assert pkg1.resources[0].url == 'http://www.hm-treasury.gov.uk/national_statistics.htm', pkg1.resources[0]
        assert pkg1.resources[0].description == 'December 2009 | hub/id/119-36345', pkg1.resources[0].description
        assert len(custody.resources) == 2, custody.resources
        assert custody.resources[0].url == 'http://www.justice.gov.uk/publications/endofcustodylicence.htm', custody.resources[0]
        assert custody.resources[0].description == 'November 2009 | hub/id/119-36836', custody.resources[0].description
        assert custody.resources[1].url == 'http://www.justice.gov.uk/publications/endofcustodylicence.htm', custody.resources[0]
        assert custody.resources[1].description == 'December 2009 | hub/id/119-36838', custody.resources[1].description
        assert pkg1.extras['date_released'] == u'2010-01-06', pkg1.extras['date_released']
        assert probation.extras['date_released'] == u'2010-01-04', probation.extras['date_released']
        assert pkg1.extras['department'] == u"Her Majesty's Treasury", pkg1.extras['department']
        assert cereals.extras['department'] == u"Department for Environment, Food and Rural Affairs", cereals.extras['department']
        assert custody.extras['department'] == u"Ministry of Justice", custody.extras['department']
        assert u"Source agency: HM Treasury" in pkg1.notes, pkg1.notes
        assert pkg1.extras['categories'] == 'Economy', pkg1.extras['category']
        assert pkg1.extras['geographic_coverage'] == '111100: United Kingdom (England, Scotland, Wales, Northern Ireland)', pkg1.extras['geographic_coverage']
        assert pkg1.extras['national_statistic'] == 'no', pkg1.extras['national_statistic']
        assert cereals.extras['national_statistic'] == 'yes', cereals.extras['national_statistic']
        assert custody.extras['national_statistic'] == 'no', custody.extras['national_statistic']
        assert 'Designation: Official Statistics not designated as National Statistics' in custody.notes
        assert pkg1.extras['geographical_granularity'] == 'UK and GB', pkg1.extras['geographical_granularity']
        assert 'Language: English' in pkg1.notes, pkg1.notes
        def check_tags(pkg, tags_list):            
            pkg_tags = [tag.name for tag in pkg.tags]
            for tag in tags_list:
                assert tag in pkg_tags, "Couldn't find tag '%s' in tags: %s" % (tag, pkg_tags)
        check_tags(pkg1, ('economics-and-finance', 'reserves', 'currency', 'assets', 'liabilities', 'gold', 'economy', 'government-receipts-and-expenditure', 'public-sector-finance'))
        check_tags(cereals, ('environment', 'farming'))
        check_tags(custody, ('public-order-justice-and-rights', 'justice-system', 'prisons'))
        assert 'Alternative title: UK Reserves' in pkg1.notes, pkg1.notes
        
        assert pkg1.extras['external_reference'] == u'ONSHUB', pkg1.extras['external_reference']
        assert 'Open Government Licence' in pkg.license.title, pkg.license.title
        assert pkg1.extras['update_frequency'] == u'monthly', pkg1.extras['update_frequency']
        assert custody.extras['update_frequency'] == u'monthly', custody.extras['update_frequency']
        assert pkg1.author == u"Her Majesty's Treasury", pkg1.author
        assert cereals.author == u'Department for Environment, Food and Rural Affairs', cereals.author
        assert custody.author == u'Ministry of Justice', custody.author

#        assert model.Group.by_name(u'ukgov') in pkg1.groups
        for pkg in (pkg1, cereals, custody):
            assert pkg.extras['import_source'].startswith('ONS'), '%s %s' % (pkg.name, pkg.extras['import_source'])


class TestOnsLoadTwice(TestLoaderBase):
    @classmethod
    def setup_class(self):
        super(TestOnsLoadTwice, self).setup_class()
        # sample_filepath(2 has the same packages as 1, but slightly updated
        for filepath in [sample_filepath(''), sample_filepath(2)]:
            importer_ = importer.OnsImporter(filepath)
            pkg_dicts = [pkg_dict for pkg_dict in importer_.pkg_dict()]
            loader = OnsLoader(self.testclient)
            res = loader.load_packages(pkg_dicts)
            assert res['num_errors'] == 0, res
        CreateTestData.flag_for_deletion([pkg_dict['name'] for pkg_dict in pkg_dicts])

    def test_packages(self):
        pkg = model.Package.by_name(u'uk_official_holdings_of_international_reserves')
        assert pkg.title == 'UK Official Holdings of International Reserves', pkg.title
        assert pkg.notes.startswith('CHANGED'), pkg.notes
        assert len(pkg.resources) == 1, pkg.resources
        assert 'CHANGED' in pkg.resources[0].description, pkg.resources


class TestOnsLoadClashTitle(TestLoaderBase):
    # two packages with the same title, both from ONS,
    # but from different departments, so must be different packages
    @classmethod
    def setup_class(self):
        super(TestOnsLoadClashTitle, self).setup_class()
        # ons items have been split into 3 files, because search needs to
        # do indexing in between
        for suffix in 'abc':
            importer_ = importer.OnsImporter(sample_filepath('3' + suffix))
            pkg_dicts = [pkg_dict for pkg_dict in importer_.pkg_dict()]
            loader = OnsLoader(self.testclient)
            self.res = loader.load_packages(pkg_dicts)
            assert self.res['num_errors'] == 0, self.res
        CreateTestData.flag_for_deletion([pkg_dict['name'] for pkg_dict in pkg_dicts])

    def test_ons_package(self):
        pkg = model.Package.by_name(u'annual_survey_of_hours_and_earnings')
        assert pkg
        assert pkg.extras.get('department') == 'UK Statistics Authority', pkg.extras.get('department')
        assert 'Office for National Statistics' in pkg.notes, pkg.notes
        assert len(pkg.resources) == 2, pkg.resources
        assert '2007 Results Phase 3 Tables' in pkg.resources[1].description, pkg.resources
        assert '2007 Pensions Results' in pkg.resources[0].description, pkg.resources

    def test_welsh_package(self):
        pkg = model.Package.by_name(u'annual_survey_of_hours_and_earnings_')
        assert pkg
        assert pkg.extras['department'] == 'Welsh Assembly Government', pkg.extras['department']
        assert len(pkg.resources) == 1, pkg.resources
        assert '2008 Results' in pkg.resources[0].description, pkg.resources


class TestOnsLoadClashSource(TestLoaderBase):
    # two packages with the same title, and department, but one not from ONS,
    # so must be different packages
    @classmethod
    def setup_class(self):
        super(TestOnsLoadClashSource, self).setup_class()

        self.clash_name = u'cereals_and_oilseeds_production_harvest'
        CreateTestData.create_arbitrary([
            {'name':self.clash_name,
             'title':'Test clash',
             'extras':{
                 'department':'Department for Environment, Food and Rural Affairs',
                 'import_source':'DECC-Jan-09',
                 },
             }
            ])
        importer_ = importer.OnsImporter(sample_filepath(''))
        CreateTestData.flag_for_deletion(self.clash_name)
        pkg_dicts = [pkg_dict for pkg_dict in importer_.pkg_dict()]
        loader = OnsLoader(self.testclient)
        self.res = loader.load_packages(pkg_dicts)
        assert self.res['num_errors'] == 0, self.res

    def test_names(self):
        pkg1 = model.Package.by_name(self.clash_name)
        assert pkg1.title == u'Test clash', pkg1.title

        pkg2 = model.Package.by_name(self.clash_name + u'_')
        assert pkg2.title == u'Cereals and Oilseeds Production Harvest', pkg2.title

class TestOnsLoadSeries(TestLoaderBase):
    @classmethod
    def setup_class(self):
        super(TestOnsLoadSeries, self).setup_class()
        for filepath in [sample_filepath('4a'), sample_filepath('4b')]:
            importer_ = importer.OnsImporter(filepath)
            pkg_dicts = [pkg_dict for pkg_dict in importer_.pkg_dict()]
            for pkg_dict in pkg_dicts:
                assert pkg_dict['title'] == 'Regional Labour Market Statistics', pkg_dict
                assert pkg_dict['extras']['agency'] == 'Office for National Statistics', pkg_dict
                assert pkg_dict['extras']['department'] == 'UK Statistics Authority', pkg_dict
                assert '2010-08-' in pkg_dict['extras']['date_released'], pkg_dict
                assert pkg_dict['extras']['date_updated'] == '', pkg_dict
            loader = OnsLoader(self.testclient)
            res = loader.load_packages(pkg_dicts)
            assert res['num_errors'] == 0, res
        CreateTestData.flag_for_deletion('regional_labour_market_statistics')

    def test_packages(self):
        pkg = model.Package.by_name(u'regional_labour_market_statistics')
        assert pkg
        assert pkg.title == 'Regional Labour Market Statistics', pkg.title
        assert pkg.extras['agency'] == 'Office for National Statistics', pkg.extras['agency']
        assert len(pkg.resources) == 9, pkg.resources
        assert_equal(pkg.extras['date_released'], '2010-08-10')
        assert_equal(pkg.extras['date_updated'], '2010-08-13')

class TestOnsLoadMissingDept(TestLoaderBase):
    # existing package to be updated has no department given (previously
    # there was no default to 'UK Statistics Authority'.
    @classmethod
    def setup_class(self):
        super(TestOnsLoadMissingDept, self).setup_class()

        self.orig_pkg_dict = {
             "name": u"measuring_subjective_wellbeing_in_the_uk",
             "title": "Measuring Subjective Wellbeing in the UK",
             "notes": "This report reviews:\n\nWhat is subjective wellbeing and why should we measure it?\n\nHow subjective wellbeing is currently measured in the UK - what subjective wellbeing questions are already being asked on major social surveys in the UK\n\nThe potential uses of subjective wellbeing data collected via these surveys\n\n\nIt concludes that subjective wellbeing is a valid construct that can be measured reliably. This is the first output of ONS' work on subjective wellbeing.\n\nSource agency: Office for National Statistics\n\nDesignation: Supporting material\n\nLanguage: English\n\nAlternative title: Working Paper: Measuring Subjective Wellbeing in the UK",
             "license_id": "ukcrown-withrights",
             "tags": ["communities", "health-well-being-and-care", "people-and-places", "societal-wellbeing", "subjective-wellbeing-subjective-well-being-objective-measures-subjective-measures", "well-being"],
             "groups": ["ukgov"],
             "extras": {"geographic_coverage": "111100: United Kingdom (England, Scotland, Wales, Northern Ireland)", "geographical_granularity": "UK and GB", "external_reference": "ONSHUB", "temporal_granularity": "", "date_updated": "", "agency": "Office for National Statistics", "precision": "", "temporal_coverage_to": "", "temporal_coverage_from": "", "national_statistic": "no", "import_source": "ONS-ons_data_7_days_to_2010-09-17", "department": 'UK Statistics Authority', "update_frequency": "", "date_released": "2010-09-14", "categories": "People and Places"},
             "resources": [{"url": "http://www.ons.gov.uk/about-statistics/measuring-equality/wellbeing/news-and-events/index.html", "format": "", "description": "2010 | hub/id/77-31166", }],
             }
        CreateTestData.create_arbitrary([self.orig_pkg_dict])

        # same data is imported, but should find record and add department
        importer_ = importer.OnsImporter(sample_filepath(5))
        self.pkg_dict = [pkg_dict for pkg_dict in importer_.pkg_dict()][0]
        loader = OnsLoader(self.testclient)
        self.res = loader.load_package(self.pkg_dict)

    def test_reload(self):
        # Check that another package has not been created
        assert self.pkg_dict['name'] == self.orig_pkg_dict['name'], self.pkg_dict['name']
        pkg1 = model.Package.by_name(self.orig_pkg_dict['name'])

        assert pkg1.extras.get('department') == u'UK Statistics Authority', pkg1.extras


class TestNationalParkDuplicate(TestLoaderBase):
    @classmethod
    def setup_class(self):
        super(TestNationalParkDuplicate, self).setup_class()
        filepath = sample_filepath(6)
        importer_ = importer.OnsImporter(filepath)
        pkg_dicts = [pkg_dict for pkg_dict in importer_.pkg_dict()]
        self.name = u'national_park_parliamentary_constituency_and_ward_level_mid-year_population_estimates_experimental'
        for pkg_dict in pkg_dicts:
            assert pkg_dict['name'] == self.name, pkg_dict['name']
            assert pkg_dict['title'] == 'National Park, Parliamentary Constituency and Ward level mid-year population estimates (experimental)', pkg_dict
            assert pkg_dict['extras']['agency'] == 'Office for National Statistics', pkg_dict
            assert pkg_dict['extras']['department'] == 'UK Statistics Authority', pkg_dict
        loader = OnsLoader(self.testclient)
        res = loader.load_packages(pkg_dicts)
        assert res['num_errors'] == 0, res
        CreateTestData.flag_for_deletion(self.name)

    def test_packages(self):
        names = [pkg.name for pkg in model.Session.query(model.Package).all()]
        assert names == [self.name], names
        pkg = model.Package.by_name(self.name)
        assert pkg
        assert len(pkg.resources) == 3, pkg.resources

class TestDeathsOverwrite(TestLoaderBase):
    @classmethod
    def setup_class(self):
        super(TestDeathsOverwrite, self).setup_class()
        self.orig_pkg_dict = {
            "name": u"weekly_provisional_figures_on_deaths_registered_in_england_and_wales",
            "title": "Weekly provisional figures on deaths registered in England and Wales",
            "version": None, "url": None, "author": "UK Statistics Authority", "author_email": None, "maintainer": None, "maintainer_email": None,
            "notes": "Weekly death figures provide provisional counts of the number of deaths registered in England and Wales in the latest four weeks for which data are available up to the end of 2009. From week one 2010 the latest eight weeks for which data are available will be published.\n\nSource agency: Office for National Statistics\n\nDesignation: National Statistics\n\nLanguage: English\n\nAlternative title: Weekly deaths",
            "license_id": "ukcrown-withrights",
            "tags": ["death", "deaths", "life-events", "life-in-the-community", "mortality-rates", "population", "weekly-deaths"],
            "groups": ["ukgov"], "extras": {
                "geographic_coverage": "101000: England, Wales",
                "geographical_granularity": "Country",
                "external_reference": "ONSHUB",
                "temporal_coverage-from": "",
                "temporal_granularity": "",
                "date_updated": "",
                "agency": "",
                "precision": "",
                "geographic_granularity": "",
                "temporal_coverage_to": "",
                "temporal_coverage_from": "",
                "taxonomy_url": "",
                "import_source": "ONS-ons_data_60_days_to_2010-09-22",
                "date_released": "2010-08-03",
                "temporal_coverage-to": "",
                "department": "UK Statistics Authority",
                "update_frequency": "",
                "national_statistic": "yes",
                "categories": "Population"},
            "resources": [
                {"url": "http://www.statistics.gov.uk/StatBase/Prep/9684.asp", "format": "", "description": "17/07/2009 | hub/id/77-27942", "hash": "", }],
            }

        CreateTestData.create_arbitrary([self.orig_pkg_dict])

        # same data is imported, but should find record and add department
        importer_ = importer.OnsImporter(sample_filepath(7))
        self.pkg_dict = [pkg_dict for pkg_dict in importer_.pkg_dict()][0]
        loader = OnsLoader(self.testclient)
        print self.pkg_dict
        self.res = loader.load_package(self.pkg_dict)
        self.name = self.orig_pkg_dict['name']

    def test_packages(self):
        names = [pkg.name for pkg in model.Session.query(model.Package).all()]
        assert names == [self.name], names
        pkg = model.Package.by_name(self.name)
        assert pkg
        assert len(pkg.resources) == 2, pkg.resources

class TestAgencyFind(TestLoaderBase):
    @classmethod
    def setup_class(self):
        super(TestAgencyFind, self).setup_class()
        self.orig_pkg_dict = {
            "name": u"national_child_measurement_programme",
            "title": "National Child Measurement Programme",
            "version": None, "url": None, "author": None, "author_email": None, "maintainer": None, "maintainer_email": None,
            "notes": "The National Child Measurement Programme weighs and measures primary school children.\r\nThis publication was formerly announced as \"National Child Measurement Programme - Statistics on Child Obesity 2008-09\" but the title has been amended to reflect suggestions from the UKSA Assessments Board.\r\nSource agency: Information Centre for Health and Social Care\r\nDesignation: National Statistics\r\nLanguage: English\r\nAlternative title: National Child Measurement Programme",
            "license_id": "uk-ogl",
            "tags": ["health", "health-and-social-care", "health-of-the-population", "lifestyles-and-behaviours", "nhs", "well-being-and-care"],
            "groups": ["ukgov"],
            "extras": {
                "geographic_coverage": "100000: England",
                "geographical_granularity": "Country",
                "external_reference": "ONSHUB",
                "temporal_coverage-from": "",
                "temporal_granularity": "",
                "date_updated": "",
                "agency": "Information Centre for Health and Social Care",
                "precision": "",
                "geographic_granularity": "",
                "temporal_coverage_to": "",
                "temporal_coverage_from": "",
                "taxonomy_url": "",
                "import_source": "ONS-ons_data_2009-12",
                "date_released": "2009-12-10",
                "temporal_coverage-to": "",
                "department": "Department of Health",
                "update_frequency": "",
                "national_statistic": "yes",
                "categories": "Health and Social Care"},
            "resources": [{"url": "http://www.ic.nhs.uk/ncmp", "format": "", "description": "England, 2008/09 School Year | hub/id/119-37085"},
                          {"url": "http://www.dh.gov.uk/en/Publichealth/Healthimprovement/Healthyliving/DH_073787", "format": "", "description": "2008 | hub/id/119-31792"},
                          {"url": "http://www.ic.nhs.uk/ncmp", "format": "", "description": "Statistics on child obesity 2007-08 | hub/id/119-31784"}],
            }

        CreateTestData.create_arbitrary([self.orig_pkg_dict])

        # same data is imported, but should find record and add department
        importer_ = importer.OnsImporter(sample_filepath(8))
        self.pkg_dict = [pkg_dict for pkg_dict in importer_.pkg_dict()][0]
        loader = OnsLoader(self.testclient)
        print self.pkg_dict
        # load package twice, to ensure reload works too
        self.res = loader.load_package(self.pkg_dict)
        self.res = loader.load_package(self.pkg_dict)
        self.name = self.orig_pkg_dict['name']
        self.num_resources_originally = len(self.orig_pkg_dict['resources'])

    def test_packages(self):
        names = [pkg.name for pkg in model.Session.query(model.Package).all()]
        assert names == [self.name], names
        pkg = model.Package.by_name(self.name)
        assert pkg
        assert_equal(len(pkg.resources), self.num_resources_originally + 1)

    def test_resources_sorted(self):
        pkg = model.Package.by_name(self.name)
        hub_ids = [int(res.description.split('-')[-1]) for res in pkg.resources]
        assert_equal(hub_ids, sorted(hub_ids))


class TestDeletedDecoyWhenAdmin(TestLoaderBase):
    @classmethod
    def setup_class(self):
        super(TestDeletedDecoyWhenAdmin, self).setup_class()
        self.tsi = TestSearchIndexer()
        self.orig_pkg_dict = {
            "name": u"quarterly_epidemiological_commentary",
            "title": "Quarterly Epidemiological Commentary",
            "version": None, "url": None, "author": None, "author_email": None, "maintainer": None, "maintainer_email": None,
            "notes": "Epidemiological analyses of Mandatory surveillance data on MRSA bacteraemia and C. difficile infection covering at least nine quarters\r\nSource agency: Health Protection Agency\r\nDesignation: Official Statistics not designated as National Statistics\r\nLanguage: English\r\nAlternative title: Quarterly Epi Commentary",
            "license_id": "uk-ogl",
            "tags": ["conditions-and-diseases", "health", "health-and-social-care", "health-of-the-population", "nhs-trust-hcai-pct-mrsa-mrsa-bacteraemia-c-difficile-c-diff-clostridium-difficile-healthcare-associa", "well-being-and-care"],
            "groups": ["ukgov"],
            "extras": {
                "geographic_coverage": "100000: England",
                "geographical_granularity": "Other",
                "external_reference": "ONSHUB",
                "temporal_coverage-from": "",
                "temporal_granularity": "",
                "date_updated": "",
                "agency": "Health Protection Agency",
                "precision": "",
                "geographic_granularity": "",
                "temporal_coverage_to": "",
                "temporal_coverage_from": "",
                "taxonomy_url": "",
                "import_source": "ONS-ons_data_7_days_to_2010-06-23",
                "date_released": "2010-06-18",
                "temporal_coverage-to": "",
                "department": "Department of Health",
                "update_frequency": "quarterly",
                "national_statistic": "no",
                "categories": "Health and Social Care"
                },
            "resources": []            
            }
        self.deleted_decoy_pkg_dict = {
            "name": u"quarterly_epidemiological_commentary_-_none",
            "title": "Quarterly Epidemiological Commentary",
            "extras": {
                "agency": "Health Protection Agency",
                "department": "Department of Health",
                },
            }
        CreateTestData.create_arbitrary([self.orig_pkg_dict])
        CreateTestData.create_arbitrary([self.deleted_decoy_pkg_dict],
                                        extra_user_names=[u'testsysadmin'])

        # make a sysadmin user
        rev = model.repo.new_revision()
        testsysadmin = model.User.by_name(u'testsysadmin')
        model.add_user_to_role(testsysadmin, model.Role.ADMIN, model.System())

        # delete decoy
        decoy_pkg = model.Package.by_name(self.deleted_decoy_pkg_dict['name'])
        assert decoy_pkg
        decoy_pkg.delete()
        model.repo.commit_and_remove()

        self.tsi.index()

        # same data is imported, but should find record and add department
        importer_ = importer.OnsImporter(sample_filepath(9))
        self.pkg_dict = [pkg_dict for pkg_dict in importer_.pkg_dict()][0]

    @classmethod
    def teardown(self):
        pass

    def test_load(self):
        user = model.User.by_name(u'testsysadmin')
        assert user
        testclient_admin = WsgiCkanClient(self.app, api_key=user.apikey)
        loader = OnsLoader(testclient_admin)
        print self.pkg_dict
        self.res = loader.load_package(self.pkg_dict)
        self.name = self.orig_pkg_dict['name']
        self.decoy_name = self.deleted_decoy_pkg_dict['name']
        self.num_resources_originally = len(self.orig_pkg_dict['resources'])

        names = [pkg.name for pkg in model.Session.query(model.Package).all()]
        assert_equal(set(names), set((self.name, self.decoy_name)))
        pkg = model.Package.by_name(self.name)
        assert pkg
        assert_equal(len(pkg.resources), self.num_resources_originally + 1)




from curate.actions import Action
from datetime import datetime

class report(Action):
    def __call__(self, tNode, inferredTriple, token, _binding, debug = False):
        dataset = self.get(self.sbind, token)
        resource = self.get(self.obind, token)

        q = """
        PREFIX http: <http://www.w3.org/2006/http#>
        SELECT DISTINCT ?code WHERE {
            ?req http:requestURI %(resource)s .
            ?req http:resp ?resp .
            ?resp http:statusCodeNumber ?code
        }
        """ % { "resource": resource.n3() }
        code = ""
        comment = ""
        try:
            for code in tNode.network.inferredFacts.query(q):
                break
        except:
            pass
        
        q = """
        PREFIX rdfs: <http://www.w3.org/2006/http#>
        PREFIX http: <http://www.w3.org/2006/http#>
        PREFIX curl: <http://eris.okfn.org/ww/2010/12/curl#>
        SELECT DISTINCT ?comment WHERE {
            { ?req http:requestURI %(resource)s .
              ?req http:resp ?resp .
              ?resp rdfs:comment ?comment } UNION
            { ?curl curl:uri %(resource)s .
              ?curl rdfs:comment ?comment }
        }
        """ % { "resource": resource.n3() }
        try:
            for comment in tNode.network.inferredFacts.query(q):
                break
        except:
            comment = "Possibly bad URI: %s" % resource.n3()

        def recode(s):
            try:
                s = s.encode("utf-8")
            except UnicodeEncodeError:
                s = s.encode("latin1")
            return s.decode("utf-8")

        timestamp = unicode(datetime.utcnow())
        print u",".join(recode(x) for x  in (timestamp, dataset, resource, code, comment))



import re

from swiss import date

from ckanext.loader import ResourceSeriesLoader

class OnsLoader(ResourceSeriesLoader):
    def __init__(self, ckanclient):
        field_keys_to_find_pkg_by = ['title', 'department']
        resource_id_prefix = 'hub/id/'
        self.resource_id_matcher = re.compile('.* \| hub\/id\/([0-9\-]*)')
        field_keys_to_expect_invariant = [
            'update_frequency', 'geographical_granularity',
            'geographic_coverage', 'temporal_granularity',
            'precision', 'url', 'taxonomy_url', 'agency',
            'author', 'author_email', 'license_id']
        synonyms = {'department': [
            ('Department for Education',
             'Department for Children, Schools and Families'),
            ]}
        super(OnsLoader, self).__init__(
            ckanclient,
            field_keys_to_find_pkg_by,
            resource_id_prefix,
            field_keys_to_expect_invariant=field_keys_to_expect_invariant,
            synonyms=synonyms
            )

    def _get_search_options(self, field_keys, pkg_dict):
        if pkg_dict['extras']['department']:
            search_options_list = super(OnsLoader, self)._get_search_options(field_keys, pkg_dict)
        else:
            # if department is blank then search against agency instead
            # (department may have been filled in manually)
            field_keys.append('agency')
            field_keys.remove('department')
            search_options_list = super(OnsLoader, self)._get_search_options(field_keys, pkg_dict)
        return search_options_list

    def _get_hub_id(self, resource):
        '''For a given resource, returns its hub id
        e.g. "April 2009 data: Experimental Statistics | hub/id/119-46440"
              gives "119-46440"
        '''
        id_match = self.resource_id_matcher.match(resource['description'])
        if not id_match:
            return None
        return id_match.groups()[0]

    def _choose_date(self, pkg1, date2_str, earlier_or_later, extra_field):
        '''From two packages, look in an extra field and return the value
        of the one which is earlier (or later).'''
        assert earlier_or_later in ('earlier', 'later')
        dates = [pkg1['extras'].get(extra_field), date2_str]
        parsed_dates = [(date.FlexiDate.from_str(date_str).as_datetime() if date_str else None) for date_str in dates]
        non_none_parsed_dates = [d for d in parsed_dates if d]
        if not non_none_parsed_dates:
            return None
        cmp_func = min if earlier_or_later == 'earlier' else max
        picked_parsed_date = cmp_func(non_none_parsed_dates)
        return dates[parsed_dates.index(picked_parsed_date)]
    
    def _merge_resources(self, existing_pkg, pkg):
        # merge date_released and date_updated fields
        pub_date = pkg['extras']['date_released']
        pkg['extras']['date_released'] = self._choose_date(existing_pkg, pub_date, 'earlier', 'date_released')
        pkg['extras']['date_updated'] = self._choose_date(existing_pkg, pub_date, 'later', 'date_updated')        
        merged_dict = super(OnsLoader, self)._merge_resources(existing_pkg, pkg)
        # sort resources by hub_id
        cmp_hub_id = lambda res1, res2: cmp(self._get_hub_id(res1),
                                                self._get_hub_id(res2))
        merged_dict['resources'] = sorted(merged_dict['resources'], cmp=cmp_hub_id)
        return merged_dict



from ckan.lib.create_test_data import CreateTestData

class PackageFixturesBase:
    def create(self, **kwargs):
        CreateTestData.create_arbitrary(self.pkgs,
                                        extra_user_names=[self.user_name],
                                        **kwargs)

    def delete(self):
        CreateTestData.delete()

class GovFixtures(PackageFixturesBase):
    user_name = 'tester'
    
    @property
    def pkgs(self):
        if not hasattr(self, '_pkgs'):
            self._pkgs = [
    {
     'name':u'private-fostering-england-2009',
     'title':u'Private Fostering',
     'notes':u'Figures on children cared for and accommodated in private fostering arrangements, England, Year ending 31 March 2009',
     'resources':[{'url':u'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000859/SFR17_2009_tables.xls',
                  'format':u'XLS',
                  'description':u'December 2009 | http://www.statistics.gov.uk/hub/id/119-36345'},
                  {'url':u'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000860/SFR17_2009_key.doc',
                  'format':u'DOC',
                  'description':u'http://www.statistics.gov.uk/hub/id/119-34565'}],
     'url':u'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000859/index.shtml',
     'author':u'DCSF Data Services Group',
     'author_email':u'statistics@dcsf.gsi.gov.uk',
     'license':u'ukcrown',
     'tags':u'children fostering',
     'extras':{
        'external_reference':u'DCSF-DCSF-0024',
        'date_released':u'2009-07-30',
        'date_updated':u'2009-07-30',
        'update_frequency':u'annual',
        'geographic_granularity':u'regional',
        'geographic_coverage':u'100000: England',
        'department':u'Department for Children, Schools and Families',
        'temporal_granularity':u'year',
        'temporal_coverage-from':u'2008-6',
        'temporal_coverage-to':u'2009-6',
        'national_statistic':u'yes',
        'precision':u'Numbers to nearest 10, percentage to nearest whole number',
        'taxonomy_url':u'',
        'agency':u'',
        'import_source':u'ONS-Jan-09',
        }
     },
    {'name':u'weekly-fuel-prices',
     'title':u'Weekly fuel prices',
     'notes':u'Latest price as at start of week of unleaded petrol and diesel.',
     'resources':[{'url':u'', 'format':u'XLS', 'description':u''}],
     'url':u'http://www.decc.gov.uk/en/content/cms/statistics/source/prices/prices.aspx',
     'author':u'DECC Energy Statistics Team',
     'author_email':u'energy.stats@decc.gsi.gov.uk',
     'license':u'ukcrown',
     'tags':u'fuel prices',
     'extras':{
        'external_reference':u'DECC-DECC-0001',
        'date_released':u'2009-11-24',
        'date_updated':u'2009-11-24',
        'update_frequency':u'weekly',
        'geographic_granularity':u'national',
        'geographic_coverage':u'111100: United Kingdom (England, Scotland, Wales, Northern Ireland)',
        'department':u'Department of Energy and Climate Change',
        'temporal_granularity':u'weeks',
        'temporal_coverage-from':u'2008-11-24',
        'temporal_coverage-to':u'2009-11-24',
        'national_statistic':u'yes',
        'import_source':u'DECC-Jan-09',
        }
     }
                ]
        return self._pkgs

class Gov3Fixtures(PackageFixturesBase):
    user_name = 'tester'

    @property
    def pkgs(self):
        if not hasattr(self, '_pkgs'):
            self._pkgs = [
    {
     'name':u'private-fostering-england-2009',
     'title':u'Private Fostering',
     'notes':u'Figures on children cared for and accommodated in private fostering arrangements, England, Year ending 31 March 2009',
     'resources':[{'url':u'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000859/SFR17_2009_tables.xls',
                  'format':u'XLS',
                  'description':u'December 2009 | http://www.statistics.gov.uk/hub/id/119-36345'},
                  {'url':u'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000860/SFR17_2009_key.doc',
                  'format':u'DOC',
                  'description':u'http://www.statistics.gov.uk/hub/id/119-34565'}],
     'url':u'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000859/index.shtml',
     'author':u'DCSF Data Services Group',
     'author_email':u'statistics@dcsf.gsi.gov.uk',
     'license':u'uk-ogl',
     'tags':u'children fostering',
     'extras':{
        'external_reference':u'DCSF-DCSF-0024',
        'date_released':u'2009-07-30',
        'date_updated':u'2009-07-30 12:30',
        'date_update_future':u'2009-07-01',
        'update_frequency':u'annual',
        'geographic_granularity':u'regional',
        'geographic_coverage':u'100000: England',
        'department':u'Department for Children, Schools and Families',
        'temporal_granularity':u'year',
        'temporal_coverage-from':u'2008-6-24 12:30',
        'temporal_coverage-to':u'2009-6',
        'national_statistic':u'yes',
        'precision':u'Numbers to nearest 10, percentage to nearest whole number',
        'mandate':u'http://www.legislation.gov.uk/id/ukpga/Eliz2/6-7/51/section/2',                
                'taxonomy_url':u'',
        'agency':u'',
        'import_source':u'ONS-Jan-09',
        }
     },
    {'name':u'weekly-fuel-prices',
     'title':u'Weekly fuel prices',
     'notes':u'Latest price as at start of week of unleaded petrol and diesel.',
     'resources':[{'url':u'', 'format':u'XLS', 'description':u''}],
     'url':u'http://www.decc.gov.uk/en/content/cms/statistics/source/prices/prices.aspx',
     'author':u'DECC Energy Statistics Team',
     'author_email':u'energy.stats@decc.gsi.gov.uk',
     'license':u'ukcrown',
     'tags':u'fuel prices',
     'extras':{
        'external_reference':u'DECC-DECC-0001',
        'date_released':u'2009-11-24',
        'date_updated':u'2009-11-24',
        'update_frequency':u'weekly',
        'geographic_granularity':u'national',
        'geographic_coverage':u'111100: United Kingdom (England, Scotland, Wales, Northern Ireland)',
        'department':u'Department of Energy and Climate Change',
        'temporal_granularity':u'weeks',
        'temporal_coverage-from':u'2008-11-24',
        'temporal_coverage-to':u'2009-11-24',
        'national_statistic':u'yes',
        'import_source':u'DECC-Jan-09',
        }
     }
                ]
        return self._pkgs

class PackageDictUtil(object):
    @classmethod
    def check_dict(cls, dict_to_check, expected_dict):
        for key, value in expected_dict.items():
            if key == 'extras':
                cls.check_dict(dict_to_check['extras'], value)
            else:
                if value:
                    assert dict_to_check[key] == value, 'Key \'%s\' should be %r not: %r' % (key, value, dict_to_check[key])
                else:
                    assert not dict_to_check.get(key), 'Key \'%s\' should have no value, not: %s' % (key, dict_to_check[key])
        unmatching_keys = set(dict_to_check.keys()) ^ set(expected_dict.keys())
        missing_keys = set(expected_dict.keys()) - set(dict_to_check.keys())
        assert not missing_keys, 'Missing keys: %r. All unmatching keys: %r' % (extra_keys, unmatching_keys)
        extra_keys = set(dict_to_check.keys()) - set(expected_dict.keys())
        assert not extra_keys, 'Keys that should not be there: %r. All unmatching keys: %r' % (extra_keys, unmatching_keys)

def teardown_module():
    assert not CreateTestData.get_all_data(), 'A test in module %r forgot to clean-up its data: %r' % (__name__, CreateTestData.get_all_data())



from setuptools import setup, find_packages

setup(
    name='ckanext-dgu',
    version='0.2',
    author='Open Knowledge Foundation',
    author_email='info@okfn.org',
    license='AGPL',
    url='http://knowledgeforge.net/ckan/',
    description='CKAN DGU extensions',
    keywords='data packaging component tool server',
    install_requires=[
        'swiss',
        'ckanclient>=0.5',
        'xlrd>=0.7.1',
        'xlwt>=0.7.2',
        #'ckanext', when it is released
    ],
    packages=find_packages(exclude=['ez_setup']),
    include_package_data=True,
    package_data={'ckan': ['i18n/*/LC_MESSAGES/*.mo']},
    entry_points="""
        [console_scripts]
        ons_loader = ckanext.dgu.ons:load
        cospread_loader = ckanext.dgu.cospread:load
        change_licenses = ckanext.dgu.scripts.change_licenses_cmd:command
        transfer_url = ckanext.dgu.scripts.transfer_url_cmd:command
        ons_analysis = ckanext.dgu.scripts.ons_analysis_cmd:command
        ofsted_fix = ckanext.dgu.scripts.ofsted_fix_cmd:command

        [ckan.forms]
        gov3 = ckanext.dgu.forms.package_gov3:get_gov3_fieldset

        [curate.actions]
        report=ckanext.dgu.curation:report
    """,
    test_suite = 'nose.collector',
)



import re

from pylons import config

from ckan.lib.helpers import json
from ckan.lib.helpers import literal
from ckan.lib.create_test_data import CreateTestData
from ckan.tests.functional.api.test_form import BaseFormsApiCase, Api1TestCase, Api2TestCase
# until (and including) ckan 1.2e test_form was in a slightly different
# location, but you need the later version

from ckanext.dgu.tests import *

class FormsApiTestCase(BaseFormsApiCase):

    @classmethod
    def setup(self):
        self.fixtures = Gov3Fixtures()
        self.fixtures.create()
        self.pkg_dict = self.fixtures.pkgs[0]
        self.package_name = self.pkg_dict['name']

    @classmethod
    def teardown(self):
        self.fixtures.delete()

    def test_get_package_create_form(self):
        form = self.get_package_create_form(package_form='gov3')
        self.assert_formfield(form, 'Package--name', '')
        self.assert_formfield(form, 'Package--title', '')
        self.assert_not_formfield(form, 'Package--version', '')
        self.assert_formfield(form, 'Package--url', '')
        self.assert_formfield(form, 'Package--notes', '')
        self.assert_formfield(form, 'Package--resources-0-url', '')
        self.assert_formfield(form, 'Package--resources-0-format', '')
        self.assert_formfield(form, 'Package--resources-0-description', '')
        self.assert_formfield(form, 'Package--resources-0-hash', '')
        self.assert_formfield(form, 'Package--resources-0-id', '')
        self.assert_formfield(form, 'Package--author', '')
        self.assert_formfield(form, 'Package--author_email', '')
        self.assert_not_formfield(form, 'Package--maintainer', '')
        self.assert_not_formfield(form, 'Package--maintainer_email', '')
        self.assert_formfield(form, 'Package--license_id', '')
        self.assert_not_formfield(form, 'Package--extras-newfield0-key', '')
        self.assert_not_formfield(form, 'Package--extras-newfield0-value', '')
        self.assert_not_formfield(form, 'Package--extras-newfield1-key', '')
        self.assert_not_formfield(form, 'Package--extras-newfield1-value', '')
        self.assert_not_formfield(form, 'Package--extras-newfield2-key', '')
        self.assert_not_formfield(form, 'Package--extras-newfield2-value', '')
        self.assert_not_formfield(form, 'Package--extras-date_update_future-key', '')
        self.assert_not_formfield(form, 'Package--extras-date_update_future-value', '')

    def test_get_package_edit_form(self):
        package = self.get_package_by_name(self.package_name)
        form = self.get_package_edit_form(package.id, package_form='gov3')
        prefix = 'Package-%s-' % package.id
        self.assert_formfield(form, prefix + 'name', package.name)
        self.assert_not_formfield(form, prefix + 'external_reference')
        self.assert_not_formfield(form, prefix + 'categories')
        expected_values = dict([(key, value) for key, value in package.extras.items()])
        expected_values['temporal_coverage-to'] = '6/2009'
        expected_values['temporal_coverage-from'] = '12:30 24/6/2008'
        expected_values['date_updated'] = '12:30 30/7/2009'
        expected_values['date_update_future'] = '1/7/2009'
        expected_values['date_released'] = '30/7/2009'
        expected_values['national_statistic'] = 'True'
        del expected_values['geographic_coverage'] # don't test here
        del expected_values['external_reference']
        del expected_values['import_source']
        for key, value in expected_values.items():
            self.assert_formfield(form, prefix + key, value)

    def test_get_package_edit_form_restrict(self):
        package = self.get_package_by_name(self.package_name)
        form = self.get_package_edit_form(package.id, package_form='gov3', restrict=1)
        prefix = 'Package-%s-' % package.id
        self.assert_not_formfield(form, prefix + 'name', package.name)
        self.assert_formfield(form, prefix + 'notes', package.notes)
        for key in ('department', 'national_statistic'):
            value = package.extras[key]
            self.assert_not_formfield(form, prefix + key, value)
        

class TestFormsApi1(Api1TestCase, FormsApiTestCase): pass

class TestFormsApi2(Api2TestCase, FormsApiTestCase): pass


class EmbeddedFormTestCase(BaseFormsApiCase):
    '''Tests the form as it would be used embedded in dgu html.'''

    def setup(self):
        self.fixtures = Gov3Fixtures()
        self.fixtures.create()
        self.pkg_dict = self.fixtures.pkgs[0]
        self.package_name = self.pkg_dict['name']
        test_user = self.get_user_by_name(unicode(self.fixtures.user_name))
        self.extra_environ = {
            'Authorization' : str(test_user.apikey)
        }
        

    def teardown(self):
        self.fixtures.delete()

    def _insert_into_field_tag(self, form_html, field_name, tag_name, tag_insertion):
        '''Finds the tag for a package field and inserts some html into it.'''
        form_html, num_replacements = re.subn(
            '(<%s[^>]* id="Package-.{0,36}-%s" .*) ?((value)|(name)=[^>]*>)' % \
            (tag_name, field_name),
            r'\1 class="disabled" readonly \2', form_html)
        assert num_replacements == 1, num_replacements
        return form_html

    def form_from_res(self, res):
        assert not "<html>" in str(res.body), "The response is an HTML doc, not just a form: %s" % str(res.body)

##        res.body = self._insert_into_field_tag(res.body, 'name', 'input', 'class="disabled" readonly')
##        res.body = self._insert_into_field_tag(res.body, 'department', 'select', 'disabled="disabled" readonly')
##        res.body = self._insert_into_field_tag(res.body, 'national_statistic', 'input', 'disabled="disabled" readonly')
        res.body = '''
<html>
  </body>
    <form method="post">
        %s
        <input type="submit">
    </form>
  </body>
</html>
''' % res.body

        return res.forms[0]

    def test_submit_package_create_form_valid(self):
        package_name = u'new_name'
        assert not self.get_package_by_name(package_name)
        form = self.get_package_create_form(package_form='gov3')
        res = self.post_package_create_form(form=form, package_form='gov3', name=package_name, department='Scotland Office', license_id='gfdl', notes='def', title='efg')
        self.assert_header(res, 'Location')
        assert not json.loads(res.body)
        self.assert_header(res, 'Location', 'http://localhost'+self.package_offset(package_name))
        pkg = self.get_package_by_name(package_name)
        assert pkg
        
    def test_submit_package_edit_form_valid(self):
        package_name = self.package_name
        pkg = self.get_package_by_name(package_name)
        new_title = u'New Title'
        form = self.get_package_edit_form(pkg.id, package_form='gov3')
        res = self.post_package_edit_form(pkg.id, form=form, title=new_title, package_form='gov3')
        # TODO work out if we need the Location header or not
#        self.assert_header(res, 'Location')
        assert not json.loads(res.body), res.body
#        self.assert_header(res, 'Location', 'http://localhost'+self.package_offset(package_name))
        pkg = self.get_package_by_name(package_name)
        assert pkg.title == new_title, pkg

    def test_submit_package_edit_form_valid_restrict(self):
        package_name = self.package_name
        pkg = self.get_package_by_name(package_name)
        new_title = u'New Title'
        form = self.get_package_edit_form(pkg.id, package_form='gov3', restrict=1)
        prefix = 'Package-%s-' % pkg.id
        self.assert_not_formfield(form, prefix + 'name', pkg.name)
        res = self.post_package_edit_form(pkg.id, form=form, title=new_title, package_form='gov3', restrict=1)
        assert not json.loads(res.body), res.body
        pkg = self.get_package_by_name(package_name)
        assert pkg.title == new_title, pkg

    def test_create_package(self):
        res = self.get_package_create_form()
        # TODO finish this test

    # TODO add other tests in from test_form.py

class TestEmbeddedFormApi1(Api1TestCase, EmbeddedFormTestCase): pass

class TestEmbeddedFormApi2(Api2TestCase, EmbeddedFormTestCase): pass

class TestGeoCoverageBug(BaseFormsApiCase, Api2TestCase):
    @classmethod
    def setup(self):
        self.user_name = u'tester1'
        self.pkg_dict = {"name": u"lichfield-councillors", "title": "Councillors", "version": None, "url": "http://www.lichfielddc.gov.uk/data", "author": "Democratic and Legal", "author_email": None, "maintainer": "Web Team", "maintainer_email": "webmaster@lichfielddc.gov.uk", "notes": "A list of Lichfield District Councillors, together with contact details, political party and committees", "license_id": "localauth-withrights", "license": "OKD Compliant::Local Authority Copyright with data.gov.uk rights", "tags": ["committees", "cool", "councillors", "democracy", "lichfield", "meetings"], "groups": ["ukgov"], "extras": {"temporal_coverage-from": "", "date_updated": "2010-03-29", "temporal_coverage_to": "", "import_source": "COSPREAD-cospread-2010-03-31mk2.csv", "geographical_granularity": "local authority", "temporal_granularity": "", "agency": "", "geographic_granularity": "", "temporal_coverage-to": "", "department": "Scotland Office", "precision": "", "temporal_coverage_from": "", "taxonomy_url": "", "mandate": "", "categories": "", "geographic_coverage": "010000: Scotland", "external_reference": "", "national_statistic": "no", "date_update_future": "", "update_frequency": "Daily", "date_released": "2009-08-01"}, "resources": [{"id": "4ef0c23f-1ebd-41c6-86a9-0f6ef81450a6", "package_id": "35697166-4600-4995-bb73-4c8ff48d52ef", "url": "http://www.lichfielddc.gov.uk/site/custom_scripts/councillors_xml.php?viewBy=name", "format": "Other XML", "description": "", "hash": "", "position": 0}]}
        CreateTestData.create_arbitrary([self.pkg_dict], extra_user_names=[self.user_name])
        self.package_name = self.pkg_dict['name']

        test_user = self.get_user_by_name(unicode(self.user_name))
        self.extra_environ = {
            'Authorization' : str(test_user.apikey)
        }

    @classmethod
    def teardown(self):
        CreateTestData.delete()

    def test_edit_coverage(self):
        package = self.get_package_by_name(self.package_name)
        form = self.get_package_edit_form(package.id, package_form='gov3')
        prefix = 'Package-%s-' % package.id
        self.assert_formfield(form, prefix + 'name', package.name)
        self.assert_formfield(form, prefix + 'geographic_coverage-england', None)
        self.assert_formfield(form, prefix + 'geographic_coverage-scotland', 'True')
        self.assert_formfield(form, prefix + 'geographic_coverage-wales', None)
        fields = {
            'geographic_coverage-england':True,
            'geographic_coverage-scotland':True,
            'geographic_coverage-wales':True,
            }
        res = self.post_package_edit_form(package.id, form=form, package_form='gov3', **fields)
        assert not json.loads(res.body)

        package = self.get_package_by_name(self.package_name)
        self.assert_equal(package.extras['geographic_coverage'], '111000: Great Britain (England, Scotland, Wales)')
        



from ckan import model
from ckan.tests import *
from ckan.tests.wsgi_ckanclient import WsgiCkanClient
from ckan.lib.create_test_data import CreateTestData

from ckanext.dgu.scripts.mass_changer import *

class TestMassChanger(TestController):
    @classmethod
    def setup_class(self):
        # create test data
        CreateTestData.create()
        username = 'annafan'
        user = model.User.by_name(unicode(username))
        assert user
        self.testclient = WsgiCkanClient(self.app, api_key=user.apikey, base_location='/api/2')

    @classmethod
    def teardown_class(self):
        CreateTestData.delete()

    def test_0_change_pkg(self):
        anna_before = self.anna.as_dict()
        war_before = self.war.as_dict()
        self.assert_equal(anna_before['license_id'], 'other-open')
        self.assert_equal(war_before['license_id'], None)

        # do the change
        new_license_id = 'test-license'        
        instructions = [
            ChangeInstruction(BasicPackageMatcher('name', 'annakarenina'),
                              BasicPackageChanger('license_id', new_license_id))
            ]
        self.mass_changer = MassChanger(self.testclient, instructions)
        self.mass_changer.run()

        # check anna has new license
        pkg = model.Package.by_name(u'annakarenina')
        anna_after = self.anna.as_dict()
        war_after = self.war.as_dict()
        self.assert_equal(anna_after['license_id'], new_license_id)
        self.assert_equal(war_after['license_id'], war_before['license_id'])

        # check no other properties have changed
        for pkg_dict_before, pkg_dict_after in [(anna_before, anna_after),
                                                (war_before, war_after)]:
            keys = set(pkg_dict_before.keys())
            assert keys == set(pkg_dict_after.keys()), set(pkg_dict_after.keys()) ^ set(pkg_dict_before.keys())
            for key in keys:
                if key not in ['license_id', 'license', 'revision_id',
                               'metadata_modified']:
                    assert pkg_dict_before[key] == pkg_dict_after[key], \
                           '%s %s: %r!=%r' % (pkg_dict_before['name'], key, pkg_dict_before[key], pkg_dict_after[key])

    def test_1_multiple_instructions(self):
        extra_field = 'genre'
        anna_before = self.anna.as_dict()
        war_before = self.war.as_dict()
        self.assert_equal(anna_before['extras'][extra_field], 'romantic novel')
        assert not war_before['extras'].has_key(extra_field)

        # do the change
        new_anna_value = 'test-anna' 
        new_value = 'test'
        instructions = [
            ChangeInstruction(BasicPackageMatcher(extra_field, 'romantic novel'),
                              BasicPackageChanger(extra_field, new_anna_value)),
            ChangeInstruction(AnyPackageMatcher(),
                              BasicPackageChanger(extra_field, new_value)),
            ]
        self.mass_changer = MassChanger(self.testclient, instructions)
        self.mass_changer.run()

        # check new licenses
        pkg = model.Package.by_name(u'annakarenina')
        anna_after = self.anna.as_dict()
        war_after = self.war.as_dict()
        self.assert_equal(anna_after['extras'][extra_field], new_anna_value)
        self.assert_equal(war_after['extras'][extra_field], new_value)

    def test_2_no_change(self):
        extra_field = 'original media'
        anna_before = self.anna.as_dict()
        war_before = self.war.as_dict()
        self.assert_equal(anna_before['extras'][extra_field], 'book')
        assert not war_before['extras'].has_key(extra_field)

        # do the change
        new_anna_value = 'test-anna' 
        new_value = 'test'
        instructions = [
            ChangeInstruction(BasicPackageMatcher('name', 'annakarenina'),
                              NoopPackageChanger()),
            ChangeInstruction(AnyPackageMatcher(),
                              BasicPackageChanger(extra_field, new_value)),
            ]
        self.mass_changer = MassChanger(self.testclient, instructions)
        self.mass_changer.run()

        # check new licenses
        pkg = model.Package.by_name(u'annakarenina')
        anna_after = self.anna.as_dict()
        war_after = self.war.as_dict()
        self.assert_equal(anna_after['extras'][extra_field], 'book')
        self.assert_equal(war_after['extras'][extra_field], new_value)

    def test_3_copy_field(self):
        anna_before = self.anna.as_dict()
        self.assert_equal(anna_before['name'], 'annakarenina')
        self.assert_equal(anna_before['title'], 'A Novel By Tolstoy')

        # do the change
        instructions = [
            ChangeInstruction(BasicPackageMatcher('name', 'annakarenina'),
                              BasicPackageChanger('title', '%(name)s'))
            ]
        self.mass_changer = MassChanger(self.testclient, instructions)
        self.mass_changer.run()

        # check anna has new license
        anna_after = self.anna.as_dict()
        self.assert_equal(anna_after['title'], 'annakarenina')

    def test_4_create_resource(self):
        pkg_before = self.anna.as_dict()
        self.assert_equal(len(pkg_before['resources']), 2)

        # do the change
        instructions = [
            ChangeInstruction(BasicPackageMatcher('name', 'annakarenina'),
                              CreateResource(url='http://res.xls',
                                             format='XLS',
                                             description='Full text'))
            ]
        self.mass_changer = MassChanger(self.testclient, instructions)
        self.mass_changer.run()

        # check change
        pkg_after = self.anna.as_dict()
        new_res = pkg_after['resources'][2]
        self.assert_equal(new_res['url'], 'http://res.xls')
        self.assert_equal(new_res['format'], 'XLS')
        self.assert_equal(new_res['description'], 'Full text')




import sys

from ckanext.api_command import ApiCommand

class MassChangerCommand(ApiCommand):
    def __init__(self, commands=None):
        usage = "% %prog [options]"
        if commands:
            usage += " {%s}" % '|'.join(commands)
        super(MassChangerCommand, self).__init__(usage=usage)

    def add_options(self):
        self.parser.add_option("-d", "--dry-run",
                          dest="dry_run",
                          action="store_true",
                          default=False,
                          help="Write no changes")
        self.parser.add_option("-f", "--force",
                          dest="force",
                          action="store_true",
                          default=False,
                          help="Don't abort rest of packages on an error")

    def command(self):
        super(MassChangerCommand, self).command()

        # now do command

def command():
    MassChangerCommand().command()




from common import ScriptError

from ckanclient import CkanApiError

class OnsAnalysis:
    def __init__(self, ckanclient):
        self.ckanclient = ckanclient
        assert self.ckanclient.api_version_get() == '1', self.ckanclient.api_version_get()

    def run(self):
        pkg_names = self.ckanclient.package_register_get()
        pkgs = []
        for pkg_name in pkg_names:
            if pkg_name.endswith('_'):
                pkg = self.ckanclient.package_entity_get(pkg_name)
                if 'ONS' in pkg['extras'].get('import_source', '') and \
                       pkg.get('state', 'active'):
                    dept = pkg['extras'].get('department')
                    try:
                        pkg_associated = self.ckanclient.package_entity_get(pkg_name.rstrip('_'))
                    except CkanApiError:
                        dept_associated = None
                    else:
                        dept_associated = pkg_associated['extras'].get('department')
                    pkgs.append(pkg)
                    print '%r\n-> %r %r' % (pkg_name, dept, dept_associated)
        print '%i packages' % len(pkgs)
        
